require 'test/unit'
require 'winding-polygon/vector'
require 'winding-polygon/point'

class CrossProductTest < Test::Unit::TestCase
  include WindingPolygon

  def test_positive
    assert 1 === Vector.new(1, 0).cross_product(Vector.new(0, 1))
    assert 1 === Vector.new(-1, 0).cross_product(Vector.new(0, -1))
    assert 0 === Vector.new(-1, 0).cross_product(Vector.new(1, 0))
    assert 0 === Vector.new(-1, 0).cross_product(Vector.new(2, 0))
  end

  def test_positive_for_latlon
    #current vertex
    p0= Point.new(-102.52676351947564,36.81219275411459)
    #previous vertex
    p1=Point.new(-115.86328125000495,34.15328996737699)
    #current edge
    v0 = Vector.new(p1.x-p0.x, p1.y-p0.y)

    #next vertex
    p2= Point.new(-104.17382812500229,40.17099871188528)
    #next edge
    v1 = Vector.new(p2.x-p0.x, p2.y-p0.y)

    assert_true v1.cross_product(v0)>0

  end

  def test_positive_for_latlon2
    #current vertex
    p0=Point.new(-103.69099150583577,44.1751011402773)
    #previous vertex
    p1= Point.new(-110.50195312500117,46.97322061018078)
    #current edge
    v0 = Vector.new(p1.x-p0.x, p1.y-p0.y)

    #next vertex
    p2= Point.new(-100.33100481371673,44.925766039368476)
    #next edge
    v1 = Vector.new(p2.x-p0.x, p2.y-p0.y)

    assert_true v1.cross_product(v0)>0
  end
  
  def test_negative
    assert(-1 === Vector.new(0, 1).cross_product(Vector.new(1, 0)))
  end

  def test_negative_for_latlon
    p0= Point.new(-81.4101562500034,35.021556836103684)
    p1=Point.new(-115.86328125000495,34.15328996737699)
    v0 = Vector.new(p1.x-p0.x, p1.y-p0.y)

    p2= Point.new(-100.3945312500027,32.46399995947513)
    v1 = Vector.new(p2.x-p0.x, p2.y-p0.y)

    assert_true v1.cross_product(v0)<0
  end

  def test_negative_for_latlon_2
    #current vertex
    p0=Point.new(-103.69099150583577,44.1751011402773)
    #previous vertex
    p1= Point.new(-110.50195312500117,46.97322061018078)
    #current edge
    v0 = Vector.new(p1.x-p0.x, p1.y-p0.y)

    #next vertex
    p2= Point.new(-109.95811865270818,42.77494310443354)
    #next edge
    v1 = Vector.new(p2.x-p0.x, p2.y-p0.y)

    assert_true v1.cross_product(v0)<0
  end
  
  def test_zero
    assert 0 === Vector.new(1, 1).cross_product(Vector.new(-2, -2))
  end

  def test_unnormalized
    assert 4 === Vector.new(1, 1).cross_product(Vector.new(-2, 2))
  end
end
